#ifndef _LOCK_H_
#define _LOCK_H_

#include <totp.hpp>
#include <NTPClient.h>
#include <WiFiUdp.h>

#include "settings.hpp"

static WiFiUDP udp;

class Lock
{
public:
    Lock(Settings *settings) : settings{settings}
    {
        timeClient = new NTPClient(udp);

        String hmac_str = settings->gettext("totp:hmac");
        int hmac_len = hmac_str.length();

        uint8_t *base32_key = new uint8_t[hmac_len];
        memcpy(base32_key, hmac_str.c_str(), hmac_len);

        TOTP(base32_key, 20, 30);
    }

    ~Lock() = default;

    void begin()
    {
        timeClient->begin();
    }

    bool check(String code)
    {
        timeClient->update();
        uint32_t newCode = getCodeFromTimestamp(timeClient->getEpochTime());
        uint32_t input = code.toInt();

        return newCode == input;
    }

private:
    NTPClient *timeClient;
    Settings *settings;
    // MQTT *mqtt;
    // TOTP *totp; // todo: convert to c++ class
};

#endif